@@ -1,5 +1,5 @@
 #ident "%W% %E% Copyright (c) 1993 Gert Doering"
-;
+
 /* type definitions, prototypes, defines needed for configuration stuff
  */
 
@@ -25,6 +25,7 @@ int get_config _PROTO(( char * conf_file, conf_data * cd,
 
 char * fgetline _PROTO(( FILE * fp ));
 void   norm_line _PROTO(( char ** line, char ** key ));
+void * conf_get_chat _PROTO(( char * line ));
 
 #ifndef ERROR
 #define ERROR -1
