@@ -1,5 +1,5 @@
-#ident "@(#)faxrec.c	1.44 94/04/09 Copyright (c) Gert Doering"
-;
+#ident "@(#)faxrec.c	1.56 94/10/14 Copyright (c) Gert Doering"
+
 /* faxrec.c - part of mgetty+sendfax
  *
  * this module is used when the modem sends a string that triggers the
@@ -12,9 +12,7 @@
  */
 
 #include <stdio.h>
-#ifndef _NOSTDLIB_H
-#include <stdlib.h>
-#endif
+#include "syslibs.h"
 #include <fcntl.h>
 #include <unistd.h>
 #include <string.h>
@@ -22,9 +20,9 @@
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/times.h>
-#include <malloc.h>
+#include <sys/stat.h>
 
-#ifndef sun
+#ifndef sunos4
 #include <sys/ioctl.h>
 #endif
 
@@ -38,10 +36,6 @@ static time_t call_done;
 
 time_t	time _PROTO(( long * tloc ));
 
-#if !defined(sun) && !defined(MEIBE)
-int	chmod _PROTO(( char * path, mode_t mode ));
-#endif
-
 /* all stuff in here was programmed according to a description of the
  * class 2 standard as implemented in the SupraFAX Faxmodem
  */
@@ -51,26 +45,7 @@ void fax_notify_mail _PROTO(( int number_of_pages ));
 void fax_notify_program _PROTO(( int number_of_pages ));
 #endif
 
-static char * faxpoll_server_file = NULL;
-
-void faxpoll_server_init _P1( (f), char * f )
-{
-    faxpoll_server_file = NULL;
-    if ( access( f, R_OK ) != 0 )
-    {
-	lprintf( L_ERROR, "cannot access/read '%s'", f );
-    }
-    else if ( fax_command( "AT+FLPL=1", "OK", 0 ) == ERROR )
-    {
-	lprintf( L_WARN, "faxpoll_server_init: no polling available" );
-	fax_hangup = 0;
-    }
-    else
-    {
-	lprintf( L_NOISE, "faxpoll_server_init: waiting for poll" );
-	faxpoll_server_file = f;
-    }
-}
+char * faxpoll_server_file = NULL;
 
 void faxrec _P1((spool_in), char * spool_in )
 {
@@ -86,21 +61,34 @@ extern  char * Device;
      * For some modems, it's necessary to switch to 19200 bps.
      */
 
+#ifdef FAX_USRobotics
+    /* the ultra smart USR modems do it in yet another way... */
+    fax_wait_for( "OK", 0 );
+#endif
+
     tio_get( STDIN, &tio );
 
-#ifdef FAX_RECEIVE_USE_B19200
+#if defined(FAX_RECEIVE_USE_B19200) || defined(FAX_USRobotics)
     tio_set_speed( &tio, B19200 );
 #endif
 
     tio_mode_raw( &tio );		/* no input or output post-*/
 					/* processing, no signals */
-    tio_set_flow_control( STDIN, &tio,
-			 (FAXREC_FLOW) & (FLOW_HARD|FLOW_XON_IN) );
     tio_set( STDIN, &tio );
 
     /* read: +FTSI:, +FDCS, OK */
 
+#ifndef FAX_USRobotics
     fax_wait_for( "OK", 0 );
+#endif
+
+    /* *now* set flow control (we could have set it earlier, but on SunOS,
+     * enabling CRTSCTS while DCD is low will make the port hang)
+     */
+    tio_set_flow_control( STDIN, &tio,
+			 (FAXREC_FLOW) & (FLOW_HARD|FLOW_XON_IN) );
+    tio_set( STDIN, &tio );
+    fax_set_flowcontrol( STDIN, (FAXREC_FLOW) & FLOW_HARD );
 
     fax_get_pages( 0, &pagenum, spool_in );
 
@@ -108,10 +96,8 @@ extern  char * Device;
     if ( faxpoll_server_file != NULL && fax_poll_req )
     {
 	lprintf( L_AUDIT, "fax poll: send %s...", faxpoll_server_file );
-	/* send page */
-	fax_send_page( faxpoll_server_file, &tio, 0 );
-	/* no more pages */
-	fax_command( "AT+FET=2", "OK", 0 );
+	/* send page, no mor pages to follow */
+	fax_send_page( faxpoll_server_file, &tio, pp_eop, 0 );
     }
 
     call_done = time(NULL);
@@ -169,6 +155,7 @@ int	ErrorCount = 0;
 int	ByteCount = 0;
 int i,j;
 extern  char * Device;
+char	DevId[3];
 
     /* call_start is only initialized if we're called from mgetty, not
      * when fax polling (sendfax) or from another getty (contrib/faxin).
@@ -187,26 +174,37 @@ extern  char * Device;
      * the "iiiiii" part will start repeating after approx. 8 years
      */
 
+    /* on some systems -- solaris2 -- the device name may look like
+     * "/dev/cub/a", so use "ba" instead of "/a" for the device id
+     */
+
+    strcpy( DevId, &Device[strlen(Device)-2] );
+    if ( DevId[0] == '/' ) DevId[0] = Device[strlen(Device)-3];
+    if ( DevId[0] == '/' ) DevId[0] = '-';
+
 #ifdef SHORT_FILENAMES
     sprintf(temp, "%s/f%c%07x%s.%02d", directory,
 		 fax_par_d.vr == 0? 'n': 'f',
 	         (int) call_start & 0xfffffff,
-	         &Device[strlen(Device)-2], pagenum );
+	         DevId, pagenum );
 #else
     /* include sender's fax id - if present - into filename */
     sprintf(temp, "%s/f%c%07x%s-", directory,
 		fax_par_d.vr == 0? 'n': 'f',
 		(int) call_start & 0xfffffff,
-		&Device[strlen(Device)]-2 );
+		DevId );
     i = strlen(temp);
-		
+
+    /* filter out characters that will give problems in the shell */
     for ( j=0; fax_remote_id[j] != 0; j++ )
     {
-         if ( fax_remote_id[j] == ' ' || fax_remote_id[j] == '/' )
+	char c = fax_remote_id[j];
+         if ( c == ' ' || c == '/' || c == '\\'|| c == '&' ||
+	      c == '(' || c == ')' || c == '>' || c == '<' )
 	 {
 	     if ( temp[i-1] != '-' ) temp[i++] = '-' ;
 	 }
-         else if ( fax_remote_id[j] != '"' ) temp[i++] = fax_remote_id[j];
+         else if ( c != '"' ) temp[i++] = c;
     }
     if ( temp[i-1] == '-' ) i--;
     sprintf( &temp[i], ".%02d", pagenum );
@@ -314,7 +312,12 @@ extern  char * Device;
 	{
 	    if ( c == DLE ) fputc( c, fax_fp );		/* DLE DLE -> DLE */
 	    else
-	    if ( c == ETX ) break;			/* DLE ETX -> end */
+	      if ( c == SUB )				/* DLE SUB -> 2x DLE */
+	    {						/* (class 2.0) */
+		fputc( DLE, fax_fp ); fputc( DLE, fax_fp );
+	    }
+	    else
+	      if ( c == ETX ) break;			/* DLE ETX -> end */
 	    
 	    WasDLE = 0;
 	}
@@ -401,10 +404,22 @@ static const char start_rcv = DC2;
 
 	if ( fax_wait_for( "OK", fd ) == ERROR ) return ERROR;
 
-	/* send command to receive next page
-	 * and to release post page response (+FPTS) to remote fax
+	/* check line count and bad line count. If page too short (less
+	 * than 50 lines) or bad line count too high (> lc/5), reject
+	 * page (+FPS=2, MPS, page bad - retrain requested)
 	 */
-	fax_send( "AT+FDR\r\n", fd );
+
+	if ( fhs_details &&
+	    ( fhs_lc < 50 || fhs_blc > (fhs_lc/10)+30 || fhs_blc > 500 ) )
+	{
+	    lprintf( L_WARN, "Page doesn't look good, request retrain (MPS)" );
+	    fax_command( "AT+FPS=2", "OK", fd );
+	}
+
+	/* send command to receive next page
+	 * and to release post page response (+FP[T]S) to remote fax
+	 */
+	fax_send( "AT+FDR", fd );
 
 	/* read: +FCFR, [+FDCS:], CONNECT */
 	/* if it was the *last* page, modem will send +FHNG:0 ->
