@@ -1,5 +1,5 @@
-#ident "@(#)gettydefs.c	1.6 94/03/13 Copyright (c) 1993 Gert Doering/Chris Lewis"
-;
+#ident "@(#)gettydefs.c	1.10 94/11/03 Copyright (c) 1993 Gert Doering/Chris Lewis"
+
 /* gettydefs.c
  *
  * Read /etc/gettydefs file, and permit retrieval of individual entries.
@@ -12,10 +12,7 @@
 #include <ctype.h>
 #include <fcntl.h>
 #include <unistd.h>
-#ifndef _NOSTDLIB_H
-# include <stdlib.h>
-#endif
-#include <malloc.h>
+#include "syslibs.h"
 
 #include "mgetty.h"
 #include "policy.h"
@@ -34,7 +31,7 @@ char * mydup _P1 ((s), register char *s)
 }
 #ifdef USE_GETTYDEFS
 
-static char gettydefs_ID[] = "@(#)gettydefs.c compiled with USE_GETTYDEFS"
+static char gettydefs_ID[] = "@(#)gettydefs.c compiled with USE_GETTYDEFS";
 
 #include "tio.h"
 
@@ -400,8 +397,8 @@ parsetermio _P2((ti, str), TIO *ti, char *str)
 			default:
 			    *p2 = *(p2+1);
 		    }
-		else if (*p2 == '^')
-		    *p2 = *(p2+1) - '@';
+		else if (*p2 == '^')		/* ^x means C-x, ^? is DEL */
+		    *p2 = (*(p2+1) == '?')? 0x7f : *(p2+1) - '@';
 
 		ti->c_cc[m->turnon] = *p2;
 	    } else
