@@ -1,9 +1,11 @@
-#ident "@(#)logfile.c	1.32 94/04/14 Copyright (c) Gert Doering"
-;
+#ident "@(#)logfile.c	1.48 94/11/05 Copyright (c) Gert Doering"
+
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <varargs.h>
+#ifndef NeXT
+# include <varargs.h>
+#endif
 #include <sys/types.h>
 #include <time.h>
 #include <errno.h>
@@ -16,13 +18,25 @@
 #ifdef SYSLOG
 #include <syslog.h>
 
-#if !defined(linux) && !defined(BSD)
+#if !defined(linux) && !defined(BSD) && !defined(SVR42) && !defined(solaris2)
 int openlog _PROTO(( char *, int, int ));
 int syslog _PROTO(( int, char *, ... ));
 #endif
 
 #endif
 
+/* on NeXTstep, we have to use this *UGLY* way to cheat varargs/stdarg
+ */
+#ifdef NeXT
+# define va_alist a1,a2,a3,a4,a5,a6,a7,a8,a9
+# define va_dcl long a1,a2,a3,a4,a5,a6,a7,a8,a9;
+# define vsprintf(buf,fmt,v) sprintf((buf),(fmt),a1,a2,a3,a4,a5,a6,a7,a8,a9)
+# define va_list int
+# define va_start(v)
+# define va_end(v)
+#endif
+
+
 static int log_level = LOG_LEVEL;  /* set default log level threshold (jcp) */
 
 static FILE * log_fp;
@@ -34,21 +48,28 @@ static char * log_program = "mgetty";
 
 extern int atexit _PROTO(( void (*)(void) ));
 
-/* Most systems have these variables but do not declare them. On thos
- * systems that _do_ declare them, it won't hurt
- */
+/* Most systems have these variables but do not declare them. On many
+   of those systems that _do_ declare them, it won't hurt */
 
 extern int sys_nerr;
+#if !defined(__NetBSD__) && !defined( __FreeBSD__ )
 extern char *sys_errlist[];
-extern int errno;
+#endif
 
 /* Interactive Unix is a little bit braindead - does not have atexit(),
  */
-#if defined(ISC) || defined(SVR4) || defined(sun) || defined(_3B1_) || \
-    defined(MEIBE) || defined(_SEQUENT_) || defined(_AIX)
+#if defined(ISC) || defined(SVR4) || defined(_3B1_) || \
+    defined(MEIBE) || defined(_SEQUENT_) || defined(_AIX) || \
+    defined(sysV68) || defined(M_XENIX)
 # define atexit(dummy) 
 #endif
 
+/* on SunOS, we can do it with on_exit()
+ */
+#ifdef sunos4
+# define atexit(func) on_exit(func, NULL)
+#endif
+
 void log_init_paths _P3 ( (l_program, l_path, l_infix),
 		           char * l_program, char * l_path, char * l_infix )
 {
@@ -123,6 +144,8 @@ int	log_fd;
 	close( log_fd );
 	pclose( pipe_fp );
     }
+
+    mail_logfile = FALSE;
 }
 
 int lputc _P2((level, ch), int level, char ch )
@@ -192,7 +215,8 @@ int     errnr;
 	    {
 		sprintf( ws, "cannot log to %s, disable logging", CONSOLE );
 		perror( ws );
-		log_level = 0;
+		log_level = -1;
+		return 0;
 	    }
 	}
 	
@@ -210,7 +234,7 @@ int     errnr;
 	fprintf( log_fp, "\n--" );
 #ifdef SYSLOG
 	/* initialize syslog logging */
-	openlog( log_program, LOG_PID, LOG_DAEMON );
+	openlog( log_program, LOG_PID, SYSLOG_FC );
 #endif
     }
 
@@ -256,9 +280,10 @@ int     errnr;
 		fclose( cons_fp );
 	    }
 	    else	/* last resort */
+		if ( !mail_logfile )
 	    {
-		mail_logfile = TRUE;
 		atexit( logmail );
+		mail_logfile = TRUE;
 	    }
 	}
     }	/* end if ( L_ERROR or L_FATAL ) */
