@@ -1,5 +1,5 @@
-#ident "@(#)sendfax.c	1.57 94/04/13 Copyright (c) Gert Doering"
-;
+#ident "@(#)sendfax.c	1.76 94/10/04 Copyright (c) Gert Doering"
+
 /* sendfax.c
  *
  * Send a Fax using a class 2 faxmodem.
@@ -12,13 +12,12 @@
 #include <string.h>
 #include <fcntl.h>
 #include <unistd.h>
-#ifndef _NOSTDLIB_H
-#include <stdlib.h>
-#endif
-#ifndef sun
+#include "syslibs.h"
+#ifndef sunos4
 #include <sys/ioctl.h>
 #endif
 #include <signal.h>
+#include <sys/types.h>
 
 #include "mgetty.h"
 #include "tio.h"
@@ -30,6 +29,10 @@
 
 char * fac_tel_no;
 boolean	verbose = FALSE;
+extern time_t	call_start;			/* for accounting */
+
+/* seems to missing nearly everywhere */
+time_t	time _PROTO(( long * tloc ));
 
 void exit_usage _P1( (program),
 		     char * program )
@@ -58,13 +61,28 @@ int fax_open_device _P2( (fax_tty, use_stdin),
     }
     else
     {
+	int tries;
+	
+	/* ignore leading "/dev/" prefix */
+	if ( strncmp( fax_tty, "/dev/", 5 ) == 0 ) fax_tty += 5;
+	
 	if ( verbose ) printf( "Trying fax device '/dev/%s'... ", fax_tty );
 
-	if ( makelock( fax_tty ) != SUCCESS )
+	tries = 0;
+	while ( makelock( fax_tty ) != SUCCESS )
 	{
-	    if ( verbose ) printf( "locked!\n" );
-	    lprintf( L_MESG, "cannot lock %s", fax_tty );
-	    return -1;
+	    if ( ++ tries < 3 )
+	    {
+	        if ( verbose ) { printf( "locked... " ); fflush( stdout ); }
+		sleep(5);
+	    }
+	    else
+	    {
+	        if ( verbose ) { printf( "locked, give up!\n" );
+				 fflush( stdout ); }
+		lprintf( L_MESG, "cannot lock %s", fax_tty );
+		return -1;
+	    }
 	}
 	
 	sprintf( device, "/dev/%s", fax_tty );
@@ -168,16 +186,43 @@ int fd;
 void fax_close _P1( (fd),
 		    int fd )
 {
-    fax_send( "AT+FCLASS=0\r\n", fd );
+    fax_send( "AT+FCLASS=0", fd );
     delay(100);
     close( fd );
     rmlocks();
 }
 
+
+/* sendfax-specific fax initializations */
+
+/* polling: set calling station ID, receiver on, local poll on */
+
+static int faxpoll_client_init _P2( (fd, cid), int fd, char * cid )
+{
+    char buf[60];
+
+    if ( modem_type == Mt_class2_0 )
+    {
+	sprintf( buf, "AT+FPI=\"%.40s\"", cid );
+	if ( mdm_command( buf, fd ) == ERROR ) return ERROR;
+        if ( mdm_command( "AT+FSP=1", fd ) == ERROR ) return ERROR;
+    }
+    else
+    {
+	sprintf( buf, "AT+FCIG=\"%.40s\"", cid );
+	if ( mdm_command( buf, fd ) == ERROR ) return ERROR;
+	if ( mdm_command( "AT+FSPL=1", fd ) == ERROR ) return ERROR;
+    }
+    if ( mdm_command( "AT+FCR=1", fd ) == ERROR ) return ERROR;
+
+    return NOERROR;
+}
+
+
 RETSIGTYPE fax_sig_goodbye _P1( (signo), int signo )
 {
-    lprintf( L_AUDIT, "failed, pid=%d, got signal %d, exiting...", 
-	     getpid(), signo );
+    lprintf( L_AUDIT, "failed, pid=%d, time=%ds, got signal %d, exiting...", 
+	     getpid(), ( time(NULL)-call_start ), signo );
     rmlocks();
     exit(15);				/* will close the fax device */
 }
@@ -200,16 +245,19 @@ char *	poll_directory = ".";			/* override with "-d" */
 static char 	fax_device_string[] = FAX_MODEM_TTYS;	/* writable! */
 char *	fax_devices = fax_device_string;	/* override with "-l" */
 int	fax_res_fine = 1;			/* override with "-n" */
+char *  modem_class = DEFAULT_MODEMTYPE;	/* override with "-C" */
+char *  fax_station_id = FAX_STATION_ID;	/* "-I <id>" */
 
 boolean	use_stdin = FALSE;			/* modem on stdin */
 
 int	tries;
 
+
     /* initialize logging */
     log_init_paths( argv[0], FAX_LOG, NULL );
     log_set_llevel( L_NOISE );
 
-    while ((opt = getopt(argc, argv, "d:vx:ph:l:nm:S")) != EOF) {
+    while ((opt = getopt(argc, argv, "d:vx:ph:l:nm:SC:I:")) != EOF) {
 	switch (opt) {
 	case 'd':	/* set target directory for polling */
 	    poll_directory = optarg;
@@ -229,9 +277,10 @@ int	tries;
 	    break;
 	case 'l':	/* set device(s) to use */
 	    fax_devices = optarg;
-	    if ( strchr( optarg, '/' ) != NULL )
+	    if ( optarg[0] == '/' &&
+		 strncmp( optarg, "/dev/", 5 ) != 0 )
 	    {
-		fprintf( stderr, "%s: -l: use device name without path\n",
+		fprintf( stderr, "%s: -l: device must be located in /dev!\n",
 		                 argv[0]);
 		exit(1);
 	    }
@@ -245,6 +294,18 @@ int	tries;
         case 'S':	/* modem on stdin */
 	    use_stdin = TRUE;
 	    break;
+	case 'C':	/* modem class */
+	    modem_class = optarg;
+	    if ( strcmp( modem_class, "cls2" ) != 0 &&
+		 strcmp( modem_class, "c2.0" ) != 0 )
+	    {
+		fprintf( stderr, "%s: warning: invalid modem class '-C %s'\n",
+			 argv[0], modem_class );
+	    }
+	    break;
+	case 'I':	/* local fax id */
+	    fax_station_id = optarg;
+	    break;
 	case '?':	/* unrecognized parameter */
 	    exit_usage(argv[0]);
 	    break;
@@ -304,10 +365,10 @@ int	tries;
 #endif
 
     if ( fax_command( "AT", "OK", fd ) == ERROR ||
-         fax_command( "AT+FCLASS=2", "OK", fd ) == ERROR )
+	 ( modem_type = fax_get_modem_type( fd, modem_class ) ) == Mt_unknown )
     {
-	lprintf( L_ERROR, "cannot set class 2 fax mode" );
-	fprintf( stderr, "%s: cannot set class 2 fax mode\n", argv[0] );
+	lprintf( L_ERROR, "cannot set modem to fax mode" );
+	fprintf( stderr, "%s: cannot set modem to fax mode\n", argv[0] );
 	fax_close( fd );
 	exit( 3 );
     }
@@ -319,9 +380,8 @@ int	tries;
     tio_set_speed( &fax_tio, FAX_SEND_SWITCHBD );
     tio_set( fd, &fax_tio );
 #endif
-	
-    sprintf( buf, "AT+FLID=\"%s\"", FAX_STATION_ID);
-    if ( fax_command( buf, "OK", fd ) == ERROR )
+
+    if ( fax_set_l_id( fd, fax_station_id ) == ERROR )
     {
 	lprintf( L_ERROR, "cannot set fax station ID" );
 	fprintf( stderr, "%s: cannot set fax station ID\n", argv[0] );
@@ -333,7 +393,7 @@ int	tries;
     {
 	if ( strncmp( extra_modem_init, "AT", 2 ) != 0 )
 	{
-	    fax_send( "AT", fd );
+	    write( fd, "AT", 2 );
 	}
 
 	if ( fax_command( extra_modem_init, "OK", fd ) == ERROR )
@@ -347,42 +407,58 @@ int	tries;
 	}
     }		/* end if (extra_modem_init != NULL) */
 
-    /* FIXME: ask modem if it can do 14400 bps / fine res. at all */
+    /* set desired resolution, maximum and minimum bit rate */
 
-    sprintf( buf, "AT+FDCC=%d,5,0,2,0,0,0,0", fax_res_fine );
-    fax_command( buf, "OK", fd );
+    /* FIXME: ask modem if it can do 14400 bps / fine res. at all */
+    fax_set_fdcc( fd, fax_res_fine, 14400, 0 );
 
 #if REVERSE
-    fax_command( "AT+FBOR=0", "OK", fd );
+    fax_set_bor( fd, 0 );
 #else
-    fax_command( "AT+FBOR=1", "OK", fd );
+    fax_set_bor( fd, 1 );
 #endif
 
     /* tell the modem if we are willing to poll faxes
      */
     if ( fax_poll_wanted )
     {
-	sprintf( buf, "AT+FCIG=\"%s\"", FAX_STATION_ID);
-	if ( fax_command( buf, "OK", fd ) == ERROR ||
-	     fax_command( "AT+FSPL=1", "OK", fd ) == ERROR )
+	if ( faxpoll_client_init( fd, FAX_STATION_ID ) == ERROR )
 	{
-	    lprintf( L_WARN, "AT+FSPL=1: cannot enable polling" );
+	    lprintf( L_WARN, "cannot enable polling" );
 	    fprintf( stderr, "Warning: polling is not possible!\n" );
 	    fax_poll_wanted = FALSE;
-	    fax_hangup = 0;	/* reset error flag */
 	}
     }
 
     /* set modem to use desired flow control type, dial out
      */
+    if ( fax_set_flowcontrol( fd, (FAXSEND_FLOW) & FLOW_HARD ) == ERROR )
+    {
+	lprintf( L_WARN, "cannot set modem flow control" );
+    }
+
+#ifdef FAX_MODEM_HANDSHAKE
+    if ( mdm_command( FAX_MODEM_HANDSHAKE, fd ) == ERROR )
+    {
+	lprintf( L_WARN, "cannot set FAX_MODEM_HANDSHAKE; ignored" );
+    }
+#endif
+
     if ( verbose ) { printf( "Dialing %s... ", fac_tel_no ); fflush(stdout); }
 
-    sprintf( buf, "AT%sD%s", FAX_MODEM_HANDSHAKE, fac_tel_no );
+    call_start = time( NULL );
+
+    sprintf( buf, "%s%s", FAX_DIAL_PREFIX, fac_tel_no );
     if ( fax_command( buf, "OK", fd ) == ERROR )
     {
-	lprintf( L_WARN, "dial failed (hangup_code=%d)", fax_hangup_code );
-	fprintf( stderr, "\n%s: dial %s failed (%s)\n", argv[0], fac_tel_no,
-		 fax_hangup_code == FHUP_BUSY? "BUSY" : "ERROR / NO CARRIER");
+	lprintf( L_AUDIT, "failed dialing, phone=\"%s\", +FHS:%d, time=%ds",
+		 fac_tel_no, fax_hangup_code, ( time(NULL)-call_start ) );
+	
+	if ( fax_hangup_code == FHUP_BUSY )
+	    printf( "dial failed (BUSY)\n" );
+	else
+	    fprintf( stderr, "\n%s: dial %s failed (ERROR / NO CARRIER)\n",
+		     argv[0], fac_tel_no );
 
 	/* end program - return codes signals kind of dial failure */
 	fax_close( fd );
@@ -409,6 +485,8 @@ int	tries;
     tries = 0;
     while ( argidx < argc )
     {
+	Post_page_messages ppm;
+	
 	/* send page header, if requested */
 	if ( fax_page_header )
 	{
@@ -423,37 +501,30 @@ int	tries;
 
 	/* send page */
 	if ( verbose ) printf( "sending '%s'...\n", argv[ argidx ] );
-	if ( fax_send_page( argv[ argidx ], &fax_tio, fd ) == ERROR ) break;
 
-	fax_page_tx_status = -1;
+	/* how to continue after page? */
 
-        /* transmit page punctuation
-	 * (three cases: more pages, last page but polling, last page at all)
-	 * then evaluate +FPTS: result code
-	 */
+	if ( argidx == argc -1 )	/* last page to send */
+	{
+	    if ( fax_poll_wanted && fax_to_poll )	/* polling! */
+	        ppm = pp_eom;		/* another doc. next (->phase B) */
+	    else
+	        ppm = pp_eop;		/* over & out (->hangup) */
+	}
+	else				/* not last page -> */
+	        ppm = pp_mps;		/* another page next */
+	
+	fax_page_tx_status = -1;	/* set by fax_send_page() */
 
-	if ( argidx == argc-1 )		/* was this the last page to send? */
-	  if ( fax_poll_wanted && fax_to_poll )
-	    fax_command( "AT+FET=1", "OK", fd );	/* end document */
-	  else
-	  {
-	    /* take care of some modems pulling cd low too soon */
-	    tio_carrier( &fax_tio, FALSE );
-#ifdef sun
-	    /* HW handshake has to be off while carrier is low */
-	    tio_set_flow_control(fd, &fax_tio, (FAXSEND_FLOW) & FLOW_XON_OUT);
-#endif
-	    tio_set( fd, &fax_tio );
-
-	    fax_command( "AT+FET=2", "OK", fd );	/* end session */
-	  }
-	else
-	    fax_command( "AT+FET=0", "OK", fd );	/* end page */
+	if ( fax_send_page( argv[ argidx ], &fax_tio, ppm, fd ) == ERROR )
+	{
+	    break;
+	}
 
 	/* after the page punctuation command, the modem
 	 * will send us a +FPTS:<ppm> page transmit status.
 	 * The ppm value is written to fax_page_tx_status by
-	 * fax_wait_for()
+	 * fax_send_page() / fax_send_ppm()
 	 * If the other side requests retransmission, do so.
 	 */
 
@@ -466,7 +537,7 @@ int	tries;
 	      fprintf( stderr, "WARNING: page bad (+FTPS:2), ignoring\n" );
 	      lprintf( L_WARN, "WARNING: +FPTS:2 ignored\n" );
 #else	      
-	      fprintf( stderr, "ERROR: page bad - retrain requested\n" );
+	      fprintf( stderr, "ERROR: RTN: page bad - retrain requested\n" );
 	      tries ++;	
 	      if ( tries >= FAX_SEND_MAX_TRIES )
 	      {
@@ -482,7 +553,7 @@ int	tries;
 	      }
 #endif	/* FAX_SEND_MAX_TRIES > 0 */
 	      break;
-	    case 3: fprintf( stderr, "WARNING: page good, but retrain requested\n" );
+	    case 3: fprintf( stderr, "WARNING: RTP: page good, but retrain requested\n" );
 		    break;
 	    case 4:
 	    case 5: fprintf( stderr, "WARNING: procedure interrupt requested - don't know how to handle it\n" );
@@ -502,8 +573,9 @@ int	tries;
 
     if ( argidx < argc || ( fax_hangup && fax_hangup_code != 0 ) )
     {
-	lprintf( L_WARN, "Failure transmitting %s: +FHNG:%2d",
-		 argv[argidx], fax_hangup_code );
+	lprintf( L_AUDIT, "failed transmitting %s: +FHS:%2d, time=%ds",
+		 argv[argidx], fax_hangup_code, ( time(NULL)-call_start ) );
+
 	fprintf( stderr, "\n%s: FAILED to transmit '%s'.\n",
 		         argv[0], argv[argidx] );
 
@@ -541,7 +613,8 @@ int	tries;
 	    if ( fax_get_pages( fd, &pagenum, poll_directory ) == ERROR )
 	    {
 		fprintf( stderr, "warning: polling failed\n" );
-		lprintf( L_WARN, "warning: polling failed!" );
+		lprintf( L_AUDIT, "failed: polling failed, +FHS:%2d, time=%ds",
+			 fax_hangup_code, ( time(NULL)-call_start ) );
 		fax_close( fd );
 		exit(12);
 	    }
@@ -550,5 +623,8 @@ int	tries;
     }
 
     fax_close( fd );
+
+    lprintf( L_AUDIT, "success, phone=\"%s\", time=%ds",
+	              fac_tel_no, ( time(NULL)-call_start ) );
     return 0;
 }
