@@ -1,11 +1,12 @@
-#ident "@(#)tio.c	1.21 94/04/23 Copyright (c) 1993 Gert Doering"
-;
+#ident "@(#)tio.c	1.38 94/10/22 Copyright (c) 1993 Gert Doering"
+
 /* tio.c
  *
  * contains routines dealing with SysV termio / POSIX termios / BSD sgtty
  *
  */
 
+#include <unistd.h>
 #include <sys/types.h>
 
 #ifdef _AIX
@@ -35,6 +36,14 @@ static char tio_compilation_type[]="@(#)tio.c compiled with BSD_SGTTY";
 #include <fcntl.h>
 #endif
 
+#ifdef sysV68
+#include <fcntl.h>
+#include <errno.h>
+#include <sys/tty.h>
+#include <sys/sxt.h>
+#include <sys/mvme332xt.h>
+#endif
+
 /* some systems do not define all flags needed later, e.g. NetBSD */
 
 #ifdef BSD
@@ -42,13 +51,24 @@ static char tio_compilation_type[]="@(#)tio.c compiled with BSD_SGTTY";
 # define IUCLC 0
 # endif
 # ifndef TAB3
-# define TAB3 OXTABS
+#  ifdef NeXT
+#   define TAB3 XTABS
+#  else
+#   define TAB3 OXTABS
+#  endif	/* !NeXT */
 # endif
 #endif
 
 /* baud rate table */
-
-struct speedtab speedtab[] = {
+static struct	speedtab {
+#ifdef POSIX_TERMIOS
+    speed_t	cbaud;
+#else
+    unsigned short cbaud;	/* baud rate, e.g. B9600 */
+#endif
+    int	 nspeed;		/* speed in numeric format */
+    char *speed;		/* speed in display format */
+} speedtab[] = {
 	{ B50,	  50,	 "50"	 },
 	{ B75,	  75,	 "75"	 },
 	{ B110,	  110,	 "110"	 },
@@ -57,14 +77,29 @@ struct speedtab speedtab[] = {
 	{ B200,	  200,	 "200"	 },
 	{ B300,	  300,	 "300"	 },
 	{ B600,	  600,	 "600"	 },
+#ifdef	B900
+	{ B900,	  900,	 "900"	},
+#endif
 	{ B1200,  1200,	 "1200"	 },
 	{ B1800,  1800,	 "1800"	 },
 	{ B2400,  2400,	 "2400"	 },
+#ifdef	B3600
+	{ B3600,  3600,	 "3600"	},
+#endif
 	{ B4800,  4800,	 "4800"	 },
+#ifdef	B7200
+	{ B7200,  7200,  "7200"	},
+#endif
 	{ B9600,  9600,	 "9600"	 },
+#ifdef	B14400
+	{ B14400, 14400, "14400" },
+#endif
 #ifdef	B19200
 	{ B19200, 19200, "19200" },
 #endif	/* B19200 */
+#ifdef	B28800
+	{ B28800, 28800, "28800" },
+#endif
 #ifdef	B38400
 	{ B38400, 38400, "38400" },
 #endif	/* B38400 */
@@ -73,6 +108,18 @@ struct speedtab speedtab[] = {
 #endif
 #ifdef	EXTB
 	{ EXTB,	  38400, "EXTB"	 },
+#endif
+#ifdef	B57600
+	{ B57600, 57600, "57600" },
+#endif
+#ifdef	B115200
+	{ B115200,115200,"115200"},
+#endif
+#ifdef B230400
+	{ B230400,230400,"230400"},
+#endif
+#ifdef B460800
+	{ B460800,460800,"460800"},
 #endif
 	{ 0,	  0,	 ""	 }
 };
@@ -105,7 +152,7 @@ int tio_get _P2((fd, t), int fd, TIO *t )
 
 int tio_set _P2( (fd, t), int fd, TIO * t)	/*!! FIXME: flags, wait */
 {
-#ifdef sun
+#ifdef sunos4
     int modem_lines;
 #endif
 #ifdef SYSV_TERMIO
@@ -119,7 +166,7 @@ int tio_set _P2( (fd, t), int fd, TIO * t)	/*!! FIXME: flags, wait */
     {
 	lprintf( L_ERROR, "tcsetattr failed" ); return ERROR;
     }
-#ifdef sun
+#ifdef sunos4
     /* On SunOS, make sure that RTS and DTR are asserted if you wanna
      * use hardware flow control
      */
@@ -130,7 +177,7 @@ int tio_set _P2( (fd, t), int fd, TIO * t)	/*!! FIXME: flags, wait */
         modem_lines |= (TIOCM_RTS | TIOCM_DTR);
         ioctl(STDIN, TIOCMSET, &modem_lines);
     }
-#endif /* sun */
+#endif /* sunos4 */
 #endif /* posix_termios */
 
 #ifdef BSD_SGTTY
@@ -142,6 +189,21 @@ int tio_set _P2( (fd, t), int fd, TIO * t)	/*!! FIXME: flags, wait */
     return NOERROR;
 }
 
+/* check whether a given speed (integer) is valid for the given system */
+int tio_check_speed _P1( (speed), int speed )
+{
+    int i;
+    for ( i=0; speedtab[i].cbaud != 0; i++ )
+    {
+	if ( speedtab[i].nspeed == speed )
+	{
+	    return speedtab[i].cbaud;
+	}
+    }
+    lprintf( L_NOISE, "speed %d not found in table", speed );
+    return -1;
+}
+
 /* set speed, do not touch the other flags */
 int tio_set_speed _P2( (t, speed ), TIO *t, int speed )
 {
@@ -207,7 +269,7 @@ void tio_mode_cbreak _P1( (t), TIO * t )
 #if defined(SYSV_TERMIO) || defined(POSIX_TERMIOS)
     t->c_oflag = 0;
     t->c_iflag &= ~( IGNCR | ICRNL | INLCR | IUCLC );
-    t->c_lflag &= ~( ICANON );
+    t->c_lflag &= ~( ICANON | ISIG );
     t->c_cc[VMIN] = 1;
     t->c_cc[VTIME]= 0;
 #else
@@ -334,7 +396,10 @@ void tio_map_cr _P2( (t, perform_mapping), TIO * t, int
 void tio_map_uclc _P2( (t, perform_mapping), TIO * t, int
 		    perform_mapping )
 {
-#if defined(SYSV_TERMIO) || defined(POSIX_TERMIOS)
+#if defined(__bsdi__) || !defined(OLCUC) || !defined(XCASE)
+    lprintf( L_WARN, "uclc mapping not available" );
+#else
+# if defined(SYSV_TERMIO) || defined(POSIX_TERMIOS)
     if ( perform_mapping )
     {
 	t->c_iflag |= IUCLC;
@@ -347,9 +412,10 @@ void tio_map_uclc _P2( (t, perform_mapping), TIO * t, int
 	t->c_oflag &= ~OLCUC;
 	t->c_lflag &= ~XCASE;
     }
-#else
-#include "not implemented yet"
-#endif
+# else
+# include "not implemented yet"
+# endif
+#endif		/* BSDI */
 }
 
 /* tio_carrier()
@@ -414,9 +480,9 @@ void tio_carrier _P2( (t, carrier_sensitive), TIO *t, int carrier_sensitive )
  *
  * WARNING: for most systems, this function will not touch the tty
  *          settings, only modify the TIO structure. On some systems,
- *          you have to use extra ioctl()s [notably SVR4 and AIX] to
- *          modify hardware flow control settings. On those systems,
- *          these calls are done immediately.
+ *          you have to use extra ioctl()s [notably SVR4, sysV68, and
+ *          AIX] to modify hardware flow control settings. On those
+ *          systems, these calls are done immediately.
  */
 
 int tio_set_flow_control _P3( (fd, t, type), int fd, TIO * t, int type )
@@ -424,6 +490,11 @@ int tio_set_flow_control _P3( (fd, t, type), int fd, TIO * t, int type )
 #ifdef SVR4
     struct termiox tix;
 #endif
+
+    lprintf( L_NOISE, "tio_set_flow_control(%s%s%s )",
+		      type & FLOW_HARD   ? " HARD": "",
+		      type & FLOW_XON_IN ? " XON_IN": "",
+		      type & FLOW_XON_OUT? " XON_OUT": "" );
     
 #if defined( SYSV_TERMIO ) || defined( POSIX_TERMIOS )
     t->c_cflag &= ~HARDWARE_HANDSHAKE;
@@ -439,8 +510,8 @@ int tio_set_flow_control _P3( (fd, t, type), int fd, TIO * t, int type )
 #include "not yet implemented"
 #endif
     /* SVR4 came up with a new method of setting h/w flow control */
-    /* unfortunately, it's broken in 4.2! */
-#if defined(SVR4) && !defined(SVR42)
+    /* unfortunately, it's broken in 4.2 and Solaris2! */
+#if defined(SVR4) && !defined(SVR42) &&!defined(solaris2)
     if (ioctl(fd, TCGETX, &tix) < 0)
     {
 	lprintf( L_ERROR, "ioctl TCGETX" ); return ERROR;
@@ -476,6 +547,36 @@ int tio_set_flow_control _P3( (fd, t, type), int fd, TIO * t, int type )
     }
 #endif			/* DEBUG */
 #endif			/* _AIX */
+
+/*
+ * sysV68 uses special ioctls, too. The following code should work for
+ * mvme332xt controllers. The mvme337 driver supports the same ioctls
+ * but has not been tested. Others may not support hardware flow
+ * control at all. Refer to the corresponding mvmeXXX(7) manpage for
+ * details.
+ */
+#ifdef sysV68
+    if ( type & FLOW_HARD ) {
+	if ( ioctl(fd, TCSETHW, 1) < 0 ) {
+		lprintf( L_ERROR, "ioctl TCSETHW on failed" );
+		return ERROR;
+	}
+    } else {
+	if ( ioctl(fd, TCSETHW, 0) < 0 ) {
+		/* We use a lower logging priority if errno is EINVAL, so
+		 * mgetty can be used on devices which do not support the
+		 * special m332xt ioctls without filling syslog with
+		 * unnecessary error messages.
+		 */
+		if (EINVAL == errno) {
+			lprintf( L_NOISE, "ioctl TCSETHW off failed" );
+		} else {
+			lprintf( L_ERROR, "ioctl TCSETHW off failed" );
+			return ERROR;
+		}
+	}
+    }
+#endif /* sysV68 */
     return NOERROR;
 }
 
@@ -499,16 +600,26 @@ int tio_toggle_dtr _P2( (fd, msec_wait), int fd, int msec_wait )
      * bring the port to some strange state where every ioctl() later
      * on simply fails - so use special "modem control" ioctl()s to
      * lower and raise DTR
+     * Strange enough, on *some* platforms, you have to pass the mctl
+     * flag word by value, on others by reference. Oh world...
      */
 #if defined(SVR4) && defined(TIOCMBIS)		/* SVR4 special */
     int mctl = TIOCM_DTR;
 
+#ifdef sun
+    if ( ioctl( fd, TIOCMBIC, &mctl ) < 0 )
+#else
     if ( ioctl( fd, TIOCMBIC, (char *) mctl ) < 0 )
+#endif
     {
 	lprintf( L_ERROR, "TIOCMBIC failed" ); return ERROR;
     }
     delay( msec_wait );
+#ifdef sun
+    if ( ioctl( fd, TIOCMBIS, &mctl ) < 0 )
+#else
     if ( ioctl( fd, TIOCMBIS, (char *) mctl ) < 0 )
+#endif
     {
 	lprintf( L_ERROR, "TIOCMBIS failed" ); return ERROR;
     }
@@ -542,7 +653,7 @@ int tio_toggle_dtr _P2( (fd, msec_wait), int fd, int msec_wait )
     /* The "standard" way of doing things - via speed = B0
      */
     TIO t, save_t;
-#ifdef sun
+#ifdef sunos4
     int modem_lines;
 #endif
     int result;
@@ -565,7 +676,7 @@ int tio_toggle_dtr _P2( (fd, msec_wait), int fd, int msec_wait )
     tio_set( fd, &t );
     delay( msec_wait );
     
-#ifdef sun
+#ifdef sunos4
     /* on SunOS, if you hangup via B0, the DTR line will *stay* low.
      * So: enable it manually again.
      */
@@ -575,8 +686,8 @@ int tio_toggle_dtr _P2( (fd, msec_wait), int fd, int msec_wait )
 #endif
     result = tio_set( fd, &save_t );
     
-#if defined(M_UNIX) && defined(MAM_BUG)
-    /* some SCO Unix variants apparently forget to raise DTR again
+#if (defined(M_UNIX) && defined(MAM_BUG)) || defined (sysV68)
+    /* some Unix variants apparently forget to raise DTR again
      * after lowering it. Reopening the port fixes it. Crude, but works.
      */
     close( open( "/dev/tty", O_RDONLY | O_NDELAY ) );
@@ -607,7 +718,7 @@ int tio_flush_queue _P2( (fd, queue), int fd, int queue )
 	return ERROR;
     }
 #endif
-#ifdef SYSV_TEMIO
+#ifdef SYSV_TERMIO
     switch ( queue )
     {
       case TIO_Q_IN:   r = ioctl( fd, TCFLSH, 0 ); break;
@@ -619,7 +730,18 @@ int tio_flush_queue _P2( (fd, queue), int fd, int queue )
     }
 #endif
 #ifdef BSD_SGTTY
-#include "not yet implemented"
+    int arg;
+    
+    switch ( queue )
+    {
+      case TIO_Q_IN:   arg = FREAD; break;
+      case TIO_Q_OUT:  arg = FWRITE; break;
+      case TIO_Q_BOTH: arg = FREAD | FWRITE; break;
+      default:
+	lprintf( L_WARN, "tio_flush_queue: invalid ``queue'' argument" );
+	return ERROR;
+    }
+    r = ioctl( fd, TIOCFLUSH, (char *) &arg );
 #endif
     if ( r != 0 ) lprintf( L_ERROR, "tio: cannot flush queue" );
 
@@ -644,7 +766,8 @@ int tio_flow _P2( (fd, restart_output), int fd, int restart_output )
                      else r = ioctl( fd, TCXONC, 0 );
 #endif
 #ifdef BSD_SGTTY
-#include "not yet supported"
+    if ( restart_output ) r = ioctl( fd, TIOCSTART, NULL );
+                     else r = ioctl( fd, TIOCSTOP, NULL );
 #endif
     if ( r != 0 ) lprintf( L_ERROR, "tio: cannot change flow ctrl state" );
 
