@@ -1,8 +1,8 @@
-#ident "@(#)g3.c	1.4 94/02/12 Copyright (c) Gert Doering"
-;
+#ident "@(#)g3.c	1.6 94/08/08 Copyright (c) Gert Doering"
+
 #include <stdio.h>
 #include <unistd.h>
-#include <malloc.h>
+#include "syslibs.h"
 #include <string.h>
 #include <fcntl.h>
 
