@@ -1,5 +1,5 @@
-#ident "@(#)ugly.h	1.1 94/02/12 Copyright (c) Gert Doering"
-;
+#ident "@(#)ugly.h	1.3 94/08/08 Copyright (c) Gert Doering"
+
 /* this module contains various macros that help you write function
  * prototypes that work both with ANSI-C and K&R C
  * macros written by Chris Lewis
@@ -18,6 +18,7 @@
 #define _P7(x,a1,a2,a3,a4,a5,a6,a7)	(a1,a2,a3,a4,a5,a6,a7)
 #define _P8(x,a1,a2,a3,a4,a5,a6,a7,a8)	(a1,a2,a3,a4,a5,a6,a7,a8)
 #define _P9(x,a1,a2,a3,a4,a5,a6,a7,a8,a9)	(a1,a2,a3,a4,a5,a6,a7,a8,a9)
+#define _P10(x,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10)	(a1,a2,a3,a4,a5,a6,a7,a8,a9,a10)
 
 #else
 
@@ -32,6 +33,7 @@
 #define _P7(x,a1,a2,a3,a4,a5,a6,a7)	x a1;a2;a3;a4;a5;a6;a7;
 #define _P8(x,a1,a2,a3,a4,a5,a6,a7,a8)	x a1;a2;a3;a4;a5;a6;a7;a8;
 #define _P9(x,a1,a2,a3,a4,a5,a6,a7,a8,a9)	x a1;a2;a3;a4;a5;a6;a7;a8;a9;
+#define _P10(x,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10)	x a1;a2;a3;a4;a5;a6;a7;a8;a9;a10;
 
 #define const
 #define volatile
