@@ -1,4 +1,4 @@
-/* $Header: /home/klaus/mgetty/voice/RCS/vpaths.c,v 1.6 1994/02/21 19:02:34 klaus Exp $ */
+/* $Header: /home/klaus/mgetty/voice/RCS/vpaths.c,v 1.7 1994/05/27 19:20:32 klaus Exp $ */
 
 #include <string.h>
 #ifndef _NOSTDLIB_H
@@ -11,7 +11,7 @@
 path_t voice_log, voice_receive_dir, voice_message_dir,
        voice_message_list, voice_backup_message, message_flag_file, 
        message_program, button_program, dtmf_program, call_program,
-       voice_answer_file;
+       voice_fft_program, voice_answer_file;
 
 static char *
 makepath _P2((rel, base), char *rel, char *base)
@@ -65,4 +65,7 @@ voice_path_init _P0(void)
 #ifdef R_CALL_PROGRAM
     call_program = makepath(R_CALL_PROGRAM, VOICE_DIR);
 #endif
+#ifdef R_FFT_PROGRAM
+    voice_fft_program = makepath(R_FFT_PROGRAM, VOICE_DIR);
+#endif
 }
